import 'dart:async';
import 'dart:convert';

import 'package:bloc/bloc.dart';
import 'package:fl_chart/fl_chart.dart';
import 'package:http/http.dart' as http;

import './bloc.dart';
import '../../classes/calculator.dart';
import '../../classes/zillow_rates.dart';

class CalculationBloc extends Bloc<CalculationEvent, CalculationState> {
  @override
  CalculationState get initialState => InitialCalculationState();

  MortgageCalculation calculation;
  FormFields controllers;
  double interestAvg;
  bool propertyTaxesMonthly = false,
      homeownersInsuranceMonthly = false,
      hoaDuesMonthly = false,
      isInterest = false;

  List<FlSpot> principalLine = [];
  List<FlSpot> interestLine = [];
  List<FlSpot> loanLine = [];
  List<double> principleNorm = [];
  List<double> interestNorm = [];
  List<double> loanNorm = [];

  @override
  Stream<CalculationState> mapEventToState(
    CalculationEvent event,
  ) async* {
    if (event is LoadInit) {
      yield* _mapSetup();
    }
    if (event is Calculate) {
      yield* _mapCalculate();
    }
    if (event is ChangeLoanType) {
      yield* _mapChangeLoanType(event);
    }
    if (event is UpdateFields) {
      yield* _mapUpdateFields();
    }
    if (event is UpdateCalculation) {
      yield* _mapUpdateCalculation(event);
    }
    if (event is MonthlyChanged) {
      yield* _mapMonthlyChanged(event);
    }
  }

  Stream<CalculationState> _mapSetup() async* {
    yield ValuesLoading();
    final rate = await fetchRate();
    if (rate != null) {
      calculation = MortgageCalculation.filled();
      calculation.interestRate = rate?.rates?.defaultRate?.rate;
      interestAvg = rate?.rates?.defaultRate?.rate;
      dispatch(Calculate());
    }
  }

  Stream<CalculationState> _mapCalculate() async* {
    calculation.calculate();
    _updateChart();
    dispatch(UpdateFields());
  }

  Stream<CalculationState> _mapChangeLoanType(ChangeLoanType event) async* {
    calculation.loanPurpose = event.purpose;
    dispatch(UpdateFields());
    //yield ValuesLoaded(calculation, controllers);
  }

  Stream<CalculationState> _mapUpdateFields() async* {
    controllers = new FormFields();
    controllers.downPaymentCalc = calculation.downPaymentCalc;
    controllers.interestAvg = interestAvg;
    controllers.loanPurpose = calculation.loanPurpose;
    controllers.interestRate = calculation.interestRate.toString();
    controllers.purchasePrice = calculation.purchasePrice.toStringAsFixed(0);
    controllers.loanAmount = calculation.loanAmount.toStringAsFixed(0);
    controllers.downPayment = calculation.downPayment.toStringAsFixed(2);
    controllers.loanTerm = calculation.loanTerm.toDouble();
    controllers.propertyTaxes = (propertyTaxesMonthly)
        ? ((calculation.propertyTaxes / 12).toStringAsFixed(2))
        : calculation.propertyTaxes.toStringAsFixed(2);
    controllers.homeownersInsurance = (homeownersInsuranceMonthly)
        ? ((calculation.homeownersInsurance / 12).toStringAsFixed(2))
        : calculation.homeownersInsurance.toStringAsFixed(2);
    controllers.hoaDues = (hoaDuesMonthly)
        ? ((calculation.hoaDues / 12).toStringAsFixed(2))
        : calculation.hoaDues.toStringAsFixed(2);
    controllers.monthlyPayment = calculation.monthlyPayment;
    controllers.pmi = calculation.pmi;
    controllers.propTaxCalc =
        calculation.purchasePrice * calculation.propertyTaxes / 1200;
    controllers.hoaCalc = calculation.hoaDues / 12;
    controllers.totalPay = calculation.totalMonthlyPayment;
    controllers.purchaseLoan = calculation.purchaseLoan;
    controllers.principalLine = principalLine;
    controllers.principleNorm = principleNorm;
    controllers.interestLine = interestLine;
    controllers.interestNorm = interestNorm;
    controllers.loanLine = loanLine;
    controllers.loanNorm = loanNorm;
    controllers.isInterest = isInterest;

    yield ValuesLoaded(calculation, controllers);
  }

  Stream<CalculationState> _mapUpdateCalculation(
      UpdateCalculation event) async* {
    if (event.reCalc == true) {
      calculation = MortgageCalculation();
    }
    calculation.loanPurpose = event.purpose;
    if (event.monthly == true) {
      propertyTaxesMonthly = true;
      homeownersInsuranceMonthly = true;
      hoaDuesMonthly = true;
    } else if (event.monthly == false) {
      propertyTaxesMonthly = false;
      homeownersInsuranceMonthly = false;
      hoaDuesMonthly = false;
    }
    for (int i = 0; i < event.fields.length; i++) {
      if (event.values[i] != null)
        switch (event.fields[i]) {
          case 'Interest Rate':
            calculation.interestRate = double.tryParse(event.values[i]);
            break;
          case 'Purchase Price':
            calculation.purchasePrice = double.tryParse(event.values[i]);
            break;
          case 'Loan Amount':
            calculation.loanAmount = double.tryParse(event.values[i]);
            break;
          case 'Down Payment':
            calculation.downPayment = double.tryParse(event.values[i]);
            break;
          case 'Property Taxes':
            if (propertyTaxesMonthly) {
              calculation.propertyTaxes = double.tryParse(event.values[i]) * 12;
            } else {
              calculation.propertyTaxes = double.tryParse(event.values[i]);
            }
            break;
          case 'Homeowners Insurance':
            if (homeownersInsuranceMonthly) {
              calculation.homeownersInsurance =
                  double.tryParse(event.values[i]) * 12;
            } else {
              calculation.homeownersInsurance =
                  double.tryParse(event.values[i]);
            }
            break;
          case 'HOA Dues':
            if (hoaDuesMonthly) {
              calculation.hoaDues = double.tryParse(event.values[i]) * 12;
            } else {
              calculation.hoaDues = double.tryParse(event.values[i]);
            }
            break;
          case 'Term':
            calculation.loanTerm = int.parse(event.values[i]) * 12;
            break;
          default:
        }
    }
    dispatch(Calculate());
  }

  Stream<CalculationState> _mapMonthlyChanged(MonthlyChanged event) async* {
    propertyTaxesMonthly = !propertyTaxesMonthly;
    homeownersInsuranceMonthly = !homeownersInsuranceMonthly;
    hoaDuesMonthly = !hoaDuesMonthly;
    dispatch(UpdateFields());
    //yield ValuesLoaded(calculation, controllers);
  }

  void _updateChart() {
    principalLine.clear();
    interestLine.clear();
    loanLine.clear();
    principleNorm.clear();
    interestNorm.clear();
    loanNorm.clear();
    final interestRate = (calculation.interestRate) / 1200;
    int month = 0;
    double principalBal = calculation.loanAmount;
    double interest = 0;
    double principal = 0;
    double term = calculation.loanTerm / 12;

    principalLine.add(FlSpot(0, 0));
    interestLine.add(FlSpot(0, 0));

    for (int i = 1; i <= (term * 12); i++) {
      month++;
      final interestMonthly = principalBal * interestRate;
      interest += interestMonthly.roundToDouble();

      final principalMonthly = calculation.monthlyPayment.roundToDouble() -
          interestMonthly.roundToDouble();
      principal += principalMonthly.roundToDouble();

      principalBal -= principalMonthly;

      if (month == 12 * (term / 10)) {
        month = 0;
        principleNorm.add(principal.roundToDouble());
        interestNorm.add(interest.roundToDouble());
        loanNorm.add(principalBal.roundToDouble());
      }
    }
    isInterest = false;
    normalize(interestNorm, interestLine);
    normalize(principleNorm, principalLine);
    if (isInterest) {
      loanLine.add((FlSpot(0, (principleNorm[9] / interestNorm[9]) * 10)));
    } else {
      loanLine.add(FlSpot(0, 10));
    }
    normalize(loanNorm, loanLine);
    dispatch(UpdateFields());
  }

  void normalize(List<double> list, List<FlSpot> list2) {
    final double loan = calculation.loanAmount;
    for (int i = 0; i < list.length; i++) {
      if (list[9] > principleNorm[9] || isInterest) {
        isInterest = true;
        final interest = interestNorm[9];
        final percent = ((list[i] / interest) * 10);
        list2.add(FlSpot(i.toDouble() + 1, percent));
      } else {
        isInterest = false;
        double percent = (list[i] / loan) * 10;
        list2.add(FlSpot(i.toDouble() + 1, percent));
      }
    }
  }
}

Future<ZillowRates> fetchRate() async {
  // Todo: Load From Local Storage First and save
  try {
    final response = await http.get(
        'https://mortgageapi.zillow.com/getCurrentRates?partnerId=RD-GBPRNSZ');
    if (response.statusCode == 200) {
      return ZillowRates.fromJson(json.decode(response.body));
    }
  } catch (e) {
    print(e);
  }
  return null;
}

class FormFields {
  double interestAvg;
  String loanPurpose;
  double loanTerm;
  String interestRate;
  String purchasePrice;
  String downPayment;
  String loanAmount;
  String propertyTaxes;
  String homeownersInsurance;
  String hoaDues;
  double monthlyPayment;
  double pmi;
  double propTaxCalc;
  double hoaCalc;
  double totalPay;
  double purchaseLoan;
  double downPaymentCalc;
  bool isInterest;
  List<FlSpot> principalLine;
  List<FlSpot> interestLine;
  List<FlSpot> loanLine;
  List<double> principleNorm;
  List<double> interestNorm;
  List<double> loanNorm;

  FormFields({
    double interestAvg = 0,
    String loanPurpose = '',
    String loanTerm = '',
    String interestRate = '',
    String purchasePrice = '',
    String downPayment = '',
    String loanAmount = '',
    String propertyTaxes = '',
    String homeownersInsurance = '',
    String hoaDues = '',
    double monthlyPayment = 0,
    double pmi = 0,
    double propTaxCalc = 0,
    double hoaCalc = 0,
    double totalPay = 0,
    double purchaseLoan = 0,
    double downPaymentCalc = 0,
    bool isInterest = false,
    List<FlSpot> principalLine,
    List<FlSpot> interestLine,
    List<FlSpot> loanLine,
    List<double> principleNorm,
    List<double> interestNorm,
    List<double> loanNorm,
  });
}
