import 'package:moor/moor.dart';
import 'package:unify_api/unify.dart' as api;

import '../database.dart';
import '../tables/core_leads.dart';

part 'core_lead.g.dart';

@UseDao(tables: [
  CoreLeadRecords
], queries: {
  'coreLeadsById':
      'SELECT * FROM coreLeads WHERE user_id = :userId AND core_lead_id = :coreLeadId LIMIT 1',
  'coreLeadsByUser':
      'SELECT * FROM coreLeads WHERE user_id = :userId ORDER BY last_name ASC'
})
class CoreLeadsDao extends DatabaseAccessor<DatabaseService>
    with _$CoreLeadsDaoMixin {
  CoreLeadsDao(DatabaseService db) : super(db);

  Future batchUpdate(String userId, List<api.CoreLead> coreLeads) async {
    final _items = <CoreLeadRecordsCompanion>[];
    final _updated = DateTime.now();
    for (var lead in coreLeads) {
      final _item = CoreLeadRecordsCompanion(
          coreLeadId: Value(lead.id),
          userId: Value(userId),
          contactId: addField<String>(lead?.contactId),
          firstName: addField<String>(lead?.firstName),
          lastName: addField<String>(lead?.lastName),
          fullName: addField<String>(lead?.fullName),
          displayName: addField<String>(lead?.displayName),
          primaryPhone: addField<String>(lead?.primaryPhone.toString()),
          secondayPhone: addField<String>(lead?.secondaryPhone.toString()),
          email: addField<String>(lead?.email),
          mailingAddress: addField<String>(lead?.mailingAddress.toString()),
          loanNumber: addField<String>(lead?.loanNumber),
          loanPurpose: addField<String>(lead?.loanPurpose),
          cofullName: addField<String>(lead?.cofullName),
          codisplayName: addField<String>(lead?.codisplayName),
          cofirstName: addField<String>(lead?.cofirstName),
          colastName: addField<String>(lead?.colastName),
          leadSource: addField<String>(lead?.leadSource),
          leadSourceType: addField<String>(lead?.leadSourceType),
          leadSourceName: addField<String>(lead?.leadSourceName),
          coprimaryPhone: addField<String>(lead?.coprimaryPhone.toString()),
          cosecondaryPhone: addField<String>(lead?.cosecondaryPhone.toString()),
          leadSourceCellPhone:
              addField<String>(lead?.leadSourceCellPhone.toString()),
          leadSourceOfficePhone:
              addField<String>(lead?.leadSourceOfficePhone.toString()),
          leadSourceEmail: addField<String>(lead?.leadSourceEmail),
          salesAgentCellPhone:
              addField<String>(lead?.salesAgentCellPhone.toString()),
          salesAgentOfficePhone:
              addField<String>(lead?.salesAgentOfficePhone.toString()),
          salesAgentEmail: addField<String>(lead?.salesAgentEmail),
          coemail: addField<String>(lead?.coemail),
          lastUpdated: Value(DateTime.now()));
      _items.add(_item);
    }
    await into(coreLeadRecords).insertAll(_items, orReplace: true);
    await cleanCoreLeads(userId, _updated);
  }

  Future<List<CoreLead>> searchCoreLeads(String userId, String query) {
    return (select(coreLeadRecords)
          ..where((l) => l.userId.equals(userId))
          ..where((l) {
            if (num.tryParse(query) != null) {
              final _num = query
                  .replaceAll('-', '')
                  .replaceAll(' ', '')
                  .replaceAll('(', '')
                  .replaceAll(')', '')
                  .replaceAll(';', '');
              return or(
                  or(
                      or(l.primaryPhone.like('%$_num%'),
                          l.secondayPhone.like('%$_num%')),
                      or(l.coprimaryPhone.like('%$_num%'),
                          l.cosecondaryPhone.like('%$_num%'))),
                  or(
                      or(
                          l.loanNumber.like('%$query%'),
                          or(l.leadSourceCellPhone.like('%$_num%'),
                              l.leadSourceOfficePhone.like('%$_num%'))),
                      or(l.salesAgentCellPhone.like('%$_num%'),
                          l.salesAgentOfficePhone.like('%$_num%'))));
            }
            return or(
                or(
                    or(l.firstName.like('%$query%'),
                        l.lastName.like('%$query%')),
                    or(l.email.like('%$query%'),
                        l.mailingAddress.like('%$query%'))),
                or(
                    or(l.loanPurpose.like('%$query%'),
                        l.coreLeadId.equals('%$query%')),
                    or(l.cofirstName.like('%$query%'),
                        l.colastName.like('%$query%'))));
          })
          ..orderBy([
            (u) => OrderingTerm(expression: u.lastName, mode: OrderingMode.asc),
            (u) => OrderingTerm(expression: u.firstName, mode: OrderingMode.asc)
          ]))
        .get();
  }

  Future<CoreLead> getCoreLead(String userId, String coreLeadId) async {
    final _coreLeads = await coreLeadsById(userId, coreLeadId);
    if (_coreLeads != null && _coreLeads.isNotEmpty) {
      return _coreLeads.first;
    }
    return null;
  }

  Future resetCoreLeadsForUser(String userId) {
    return (delete(coreLeadRecords)..where((c) => c.userId.equals(userId)))
        .go();
  }

  Future cleanCoreLeads(
    String userId,
    DateTime lastUpdated,
  ) {
    return (delete(coreLeadRecords)
          ..where((t) => t.userId.equals(userId))
          ..where((t) => t.lastUpdated.isBiggerThanValue(lastUpdated)))
        .go();
  }
}
