import 'package:moor/moor.dart';
import 'package:unify_api/unify.dart' as api;
import 'package:flutter/foundation.dart';

import '../../../classes/enum_list.dart';
import '../database.dart';
import '../tables/dashboard.dart';

part 'dashboard.g.dart';

@UseDao(tables: [
  DashboardRecords
], queries: {
  'alertsById':
      'SELECT * FROM dashboardAlerts WHERE user_id = :userId AND alert_id = :alertId LIMIT 1',
  'alertsByFolder':
      'SELECT * FROM dashboardAlerts WHERE user_id = :userId AND folder = :folder ORDER BY last_name ASC'
})
class DashboardDao extends DatabaseAccessor<DatabaseService>
    with _$DashboardDaoMixin {
  DashboardDao(DatabaseService db) : super(db);

  Future batchUpdate(
    String userId,
    List<api.DashboardAlert> dashboardAlerts, {
    DashboardAlertType folder = DashboardAlertType.all,
    List<api.DashboardPartners> partnerAlerts,
  }) async {
    final _items = <DashboardRecordsCompanion>[];
    final _updated = DateTime.now();
    if (partnerAlerts != null && partnerAlerts.isNotEmpty) {
      for (var alert in partnerAlerts) {
        final _item = DashboardRecordsCompanion(
            alertId: Value(alert?.id),
            userId: Value(userId),
            folder: addField(describeEnum(folder),
                fallback: describeEnum(DashboardAlertType.all)),
            contactId: addField<String>(alert?.contactId),
            loanId: addField<String>(alert?.loanId),
            firstName: addField<String>(alert?.firstName),
            lastName: addField<String>(alert?.lastName),
            email: addField<String>(alert?.email),
            cellPhone: addField<String>(alert?.cellPhone),
            homePhone: addField<String>(alert?.homePhone),
            officePhone: addField<String>(alert?.officePhone),
            //interestRate: addField<double>(alert?.interestRate),
            otherInfo: addField<String>(alert?.otherInfo),
            marketingPartnerId: addField<String>(alert?.marketingPartnerId),
            branchId: addField<String>(alert?.branchId),
            company: addField<String>(alert?.company),
            loanNumber: addField<String>(alert?.loanNumber),
            alertType: addField<String>(alert?.alertType),
            alertPriority: addField<String>(alert?.alertPriority),
            dateCreated: addField<DateTime>(alert?.dateCreated),
            dateDeleted: addField<DateTime>(alert?.dateDeleted),
            deleted: addField<bool>(alert?.deleted),
            lastUpdated: Value(DateTime.now()));
        _items.add(_item);
      }
    } else {
      for (var alert in dashboardAlerts) {
        final _item = DashboardRecordsCompanion(
            alertId: Value(alert?.id),
            userId: Value(userId),
            folder: addField(describeEnum(folder),
                fallback: describeEnum(DashboardAlertType.all)),
            contactId: addField<String>(alert?.contactId),
            loanId: addField<String>(alert?.loanId),
            firstName: addField<String>(alert?.firstName),
            lastName: addField<String>(alert?.lastName),
            email: addField<String>(alert?.email),
            cellPhone: addField<String>(alert?.cellPhone),
            homePhone: addField<String>(alert?.homePhone),
            officePhone: addField<String>(alert?.officePhone),
            interestRate: addField<double>(alert?.interestRate),
            otherInfo: addField<String>(alert?.otherInfo),
            // marketingPartnerId: addField<String>(alert?.marketingPartnerId),
            // branchId: addField<String>(alert?.branchId),
            // company: addField<String>(alert?.company),
            loanNumber: addField<String>(alert?.loanNumber),
            alertType: addField<String>(alert?.alertType),
            alertPriority: addField<String>(alert?.alertPriority),
            dateCreated: addField<DateTime>(alert?.dateCreated),
            dateDeleted: addField<DateTime>(alert?.dateDeleted),
            deleted: addField<bool>(alert?.deleted),
            lastUpdated: Value(DateTime.now()));
        _items.add(_item);
      }
    }
    await into(dashboardRecords).insertAll(_items, orReplace: true);
    await cleanDashoard(userId, _updated);
  }

  Future<List<Dashboard>> searchAlerts(String userId, String query,
      {DashboardAlertType folder = DashboardAlertType.all}) {
    return (select(dashboardRecords)
          ..where((l) => l.userId.equals(userId))
          //..where((l) => l.folder.equals(describeEnum(folder)))
          ..where((l) {
            if (num.tryParse(query) != null) {
              final _num = query
                  .replaceAll('-', '')
                  .replaceAll(' ', '')
                  .replaceAll('(', '')
                  .replaceAll(')', '')
                  .replaceAll(';', '');
              return or(
                or(l.loanNumber.like('%$query%'), l.cellPhone.like('%$_num%')),
                or(l.homePhone.like('%$_num%'), l.officePhone.like('%$_num%')),
              );
            }
            return or(
                or(
                    l.email.like('%$query%'),
                    or(l.firstName.like('%$query%'),
                        l.lastName.like('%$query%'))),
                or(l.alertType.like('%$query%'),
                    l.alertPriority.like('%$query%')));
          })
          ..orderBy([
            (u) => OrderingTerm(expression: u.lastName, mode: OrderingMode.asc),
            (u) =>
                OrderingTerm(expression: u.firstName, mode: OrderingMode.asc),
          ]))
        .get();
  }

  Future<Dashboard> getLastAlert(
      String userId, DashboardAlertType folder) async {
    final _items = await (select(dashboardRecords)
          ..where((l) => l.userId.equals(userId))
          ..where((l) => l.folder.equals(describeEnum(folder)))
          ..orderBy([
            (u) => OrderingTerm(
                expression: u.lastUpdated, mode: OrderingMode.desc),
          ]))
        .get();
    if (_items != null && _items.isNotEmpty) {
      return _items.first;
    }
    return null;
  }

  Future<Dashboard> getAlert(String userId, String alertId) async {
    final _alerts = await alertsById(userId, alertId);
    if (_alerts != null && _alerts.isNotEmpty) {
      return _alerts.first;
    }
    return null;
  }

  Future resetDashboardForUser(String userId) {
    return (delete(dashboardRecords)..where((c) => c.userId.equals(userId)))
        .go();
  }

  Future cleanDashoard(String userId, DateTime lastUpdated,
      {DashboardAlertType folder = DashboardAlertType.all}) {
    return (delete(dashboardRecords)
          ..where((l) => l.userId.equals(userId))
          ..where((l) => l.lastUpdated.isBiggerThanValue(lastUpdated))
          ..where((l) => l.folder.equals(describeEnum(folder))))
        .go();
  }

  Future deleteAlert(String userId, String alertId) {
    return (delete(dashboardRecords)
          ..where((l) => l.userId.equals(userId))
          ..where((l) => l.alertId.equals(alertId)))
        .go();
  }
}
