import 'package:moor/moor.dart';
import 'package:unify_api/unify.dart' as api;

import '../database.dart';
import '../tables/leads.dart';

part 'lead.g.dart';

@UseDao(tables: [
  LeadRecords
], queries: {
  'leadsById':
      'SELECT * FROM leads WHERE user_id = :userId AND lead_id = :leadId LIMIT 1',
  'leadsByUser':
      'SELECT * FROM leads WHERE user_id = :userId ORDER BY last_name ASC'
})
class LeadsDao extends DatabaseAccessor<DatabaseService> with _$LeadsDaoMixin {
  LeadsDao(DatabaseService db) : super(db);

  Future batchUpdate(String userId, List<api.Lead> leads) async {
    final _items = <LeadRecordsCompanion>[];
    final _updated = DateTime.now();
    for (var lead in leads) {
      final _item = LeadRecordsCompanion(
          leadId: Value(lead.id),
          userId: Value(userId),
          contactId: addField<String>(lead?.contactId),
          firstName: addField<String>(lead?.firstName),
          lastName: addField<String>(lead?.lastName),
          cellPhone: addField<String>(lead?.cellPhone.toString()),
          homePhone: addField<String>(lead?.homePhone.toString()),
          officePhone: addField<String>(lead?.officePhone.toString()),
          email: addField<String>(lead?.email),
          lastActivity: addField<String>(lead?.lastActivity),
          dateCreated: addField<DateTime>(lead?.dateCreated),
          lastUpdated: Value(DateTime.now()));
      _items.add(_item);
    }
    await into(leadRecords).insertAll(_items, orReplace: true);
    await cleanLeads(userId, _updated);
  }

  Future<List<Lead>> searchLeads(String userId, String query,
      {LeadScope filter}) {
    final scope = filter ?? LeadScope();
    return (select(leadRecords)
          ..where((l) => l.userId.equals(userId))
          ..where((l) {
            final _scopes = <Expression<bool, BoolType>>[];
            if (scope.email) {
              _scopes.add(l.email.like('%$query%'));
            }
            if (scope.firstName) {
              _scopes.add(l.firstName.like('%$query%'));
            }
            if (scope.lastName) {
              _scopes.add(l.lastName.like('%$query%'));
            }
            if (scope.lastActivity) {
              _scopes.add(l.lastActivity.like('%$query%'));
            }
            final _num = query
                .replaceAll('-', '')
                .replaceAll(' ', '')
                .replaceAll('(', '')
                .replaceAll(')', '')
                .replaceAll(';', '');
            if (scope.cellPhone) {
              _scopes.add(l.cellPhone.like('%$_num%'));
            }
            if (scope.homePhone) {
              _scopes.add(l.homePhone.like('%$_num%'));
            }
            if (scope.officePhone) {
              _scopes.add(l.officePhone.like('%$_num%'));
            }

            Expression<bool, BoolType> _exp, _a, _b;
            while (_scopes.isNotEmpty) {
              _a = _scopes.removeAt(0);
              _b = _scopes.length > 1 ? _scopes.removeAt(1) : null;
              _exp = ExpSearch(a: _a, b: _b, c: _exp).exp;
            }
            return _exp;
          })
          ..orderBy([
            (u) => OrderingTerm(expression: u.lastName, mode: OrderingMode.asc),
            (u) =>
                OrderingTerm(expression: u.firstName, mode: OrderingMode.asc),
          ]))
        .get();
  }

  Future<Lead> getLastLead(String userId) async {
    final _item = await (select(leadRecords)
          ..where((l) => l.userId.equals(userId))
          ..orderBy([
            (u) => OrderingTerm(
                expression: u.lastUpdated, mode: OrderingMode.desc),
          ]))
        .get();
    if (_item != null && _item.isNotEmpty) {
      return _item.first;
    }
    return null;
  }

  Future<Lead> getLead(String userId, String leadId) async {
    final _leads = await leadsById(userId, leadId);
    if (_leads != null && _leads.isNotEmpty) {
      return _leads.first;
    }
    return null;
  }

  Future insertDetails(String userID, String leadID, String details) async {
    final lead = await leadsById(userID, leadID);

    if (lead != null && lead.isNotEmpty) {
      final _lead = lead.first
          .copyWith(leadDetails: details, lastUpdated: DateTime.now());
      return into(leadRecords).insert(_lead, orReplace: true);
    }
  }

  Future resetLeadsForUser(String userId) {
    return (delete(leadRecords)..where((c) => c.userId.equals(userId))).go();
  }

  Future cleanLeads(String userId, DateTime lastUpdated) async {
    return await (delete(leadRecords)
          ..where((l) => l.userId.equals(userId))
          ..where((l) => l.lastUpdated.isBiggerThanValue(lastUpdated)))
        .go();
  }

  Future deleteLead(String userId, String leadId) async {
    return (delete(leadRecords)
          ..where((l) => l.userId.equals(userId))
          ..where((l) => l.leadId.equals(leadId)))
        .go();
  }
}

class LeadScope {
  LeadScope({
    this.email = true,
    this.cellPhone = true,
    this.homePhone = true,
    this.officePhone = true,
    this.firstName = true,
    this.lastName = true,
    this.lastActivity = true,
  });
  bool email,
      cellPhone,
      homePhone,
      officePhone,
      firstName,
      lastName,
      lastActivity;
}
