import 'package:flutter/foundation.dart';
import 'package:moor/moor.dart' hide Column;
import 'package:unify_api/unify.dart' as api;

import '../../utils/utils.dart';
import 'dao/contact.dart';
import 'dao/contact_group.dart';
import 'dao/core_lead.dart';
import 'dao/dashboard.dart';
import 'dao/favorites.dart';
import 'dao/group_item.dart';
import 'dao/home_dashboard.dart';
import 'dao/lead.dart';
import 'dao/lead_group.dart';
import 'dao/loan.dart';
import 'dao/rate.dart';
import 'dao/video.dart';
import 'dao/video_folders.dart';
import 'tables/contact_group.dart';
import 'tables/contacts.dart';
import 'tables/core_leads.dart';
import 'tables/dashboard.dart';
import 'tables/favorites.dart';
import 'tables/group_item.dart';
import 'tables/home.dart';
import 'tables/index.dart';
import 'tables/lead_group.dart';
import 'tables/leads.dart';
import 'tables/loan.dart';
import 'tables/rates.dart';
import 'tables/video_folder.dart';
import 'tables/videos.dart';

part 'database.g.dart';

@UseMoor(
  tables: [
    UserRecords,
    LoanRecords,
    ContactRecords,
    LeadRecords,
    DashboardRecords,
    VideoRecords,
    CoreLeadRecords,
    CalendarRecords,
    LogRecords,
    AlertRecords,
    HomeRecords,
    RatesRecords,
    FavoriteRecords,
    LeadGroupRecords,
    GroupItemRecords,
    VideoFolderRecords,
    ContactGroupRecords
  ],
  daos: [
    LoansDao,
    ContactsDao,
    LeadsDao,
    DashboardDao,
    VideoDao,
    CoreLeadsDao,
    HomeDao,
    RateDao,
    FavoritesDao,
    LeadGroupDao,
    GroupItemDao,
    VideoFolderDao,
    ContactGroupDao,
  ],
  queries: {
    'userById': 'SELECT * FROM users WHERE user_i_d = :id LIMIT 1',
  },
)
class DatabaseService extends _$DatabaseService {
  DatabaseService(QueryExecutor e) : super(e);

  @override
  int get schemaVersion => 3;

  @override
  MigrationStrategy get migration => MigrationStrategy(
        onCreate: (m) async {
          return await m.createAllTables();
        },
        onUpgrade: (m, from, to) async {
          switch (from) {
            case 3:
              // await m.addColumn(userRecords, userRecords.domain);
              break;
            default:
          }
          // m.deleteTable('loans');
          return await m.createAllTables();
        },
      );

  bool checkField(Map<String, dynamic> _data, String field) =>
      _data.containsKey(field) && _data[field] != null;

  Future<User> getUser(String id) async {
    final _users = await userById(id);
    if (_users != null && _users.isNotEmpty) {
      return _users.first;
    }
    return null;
  }

  Future<User> insertUser(
      api.UnifyClient client, api.AccountInfo user, String domain) async {
    final _item = UserRecordsCompanion(
      userID: Value(user.id),
      domain: Value(domain),
      username: addField<String>(client?.email),
      password: addField<String>(client?.password),
      token: addField<String>(client?.token?.accessToken),
      firstName: addField<String>(user?.firstName),
      lastName: addField<String>(user?.lastName),
      profileImage: addField<String>(user?.profileImage),
      title: addField<String>(user?.title),
      email: addField<String>(user?.email),
      facebookUrl: addField<String>(user?.facebookUrl),
      twitterUrl: addField<String>(user?.twitterUrl),
      linkedInUrl: addField<String>(user?.linkedInUrl),
      websiteUrl: addField<String>(user?.websiteUrl),
      companyId: addField<String>(user?.companyId),
      companyName: addField<String>(user?.companyName),
      companyImage: addField<String>(user?.companyImage),
      kalturaId: addField<String>(user?.kalturaId),
      licenseNumber: addField<String>(user?.licenseNumber),
      losLicenseNumber: addField<String>(user?.losLicenseNumber),
      videoEnabled: addField<bool>(user?.videoEnabled, fallback: false),
      primaryPhone: addField<String>(user?.primaryPhone?.toString()),
      secondaryPhone: addField<String>(user?.secondaryPhone?.toString()),
      lastLogin: addField<DateTime>(DateTime.now()),
    );
    await into(userRecords).insert(_item, orReplace: true);
    return getUser(user.id);
  }

  Future deleteUser(User user) {
    return (delete(userRecords)..where((l) => l.userID.equals(user.userID)))
        .delete(user);
  }

  Future<User> getLastUser() async {
    final _users = await (select(userRecords)
          ..where((l) => l.lastLogin.isBiggerOrEqualValue(
              DateTime.now().subtract(Duration(days: kValidUserDays))))
          ..orderBy([
            (u) =>
                OrderingTerm(expression: u.lastLogin, mode: OrderingMode.desc),
          ]))
        .get();
    if (_users != null && _users.isNotEmpty) {
      return _users.first;
    }
    return null;
  }

  Stream<List<User>> watchOtherUsers(String userID) {
    return (select(userRecords)
          ..where((l) => not(l.userID.equals(userID)))
          ..orderBy([
            (u) =>
                OrderingTerm(expression: u.lastLogin, mode: OrderingMode.desc),
          ]))
        .watch();
  }
}

Value<T> addField<T>(T val, {T fallback}) {
  Value<T> _fallback;
  if (fallback != null) {
    _fallback = Value<T>(fallback);
  }
  if (val == null) {
    return _fallback ?? Value.absent();
  }
  if (val is String && (val == 'null' || val == 'Null')) {
    return _fallback ?? Value.absent();
  }
  return Value(val);
}

enum SortField {
  lastName,
  firstName,
  email,
  cellPhone,
  homePhone,
  officePhone,
  dateCreated,
  lastUpdated,
  lastActivity,
}

class ExpSearch {
  const ExpSearch({
    this.a,
    this.b,
    this.c,
  });
  final Expression<bool, BoolType> a, b, c;

  Expression<bool, BoolType> get exp {
    if (c != null) {
      Expression<bool, BoolType> _a, _b;
      if (b == null || a == null) {
        _a = a ?? b;
        return or(_a, c);
      } else {
        _a = a;
        _b = b;
        final _ep = or(_a, _b);
        return or(_ep, c);
      }
    }
    if (b == null || a == null) {
      return a ?? b;
    }
    return or(a, b);
  }
}
